#
# fmtcheck.rb
#
#   Copyright (c) 2000 Minero Aoki <aamine@dp.u-netsurf.ne.jp>
#
#   This program is free software.
#   you can distribute/modify this program under the terms of
#   the GNU Lesser General Public License version 2 or later.
#

require 'strscan'


module EST

  module FomatCheck

    module_function

    def check( ent, fmt )
      fmt = fmt.gsub( /\s/, '' )
      chk = scan_h( StringScanner.new fmt )
      chk.chk ent
    end

    def scan_hash( fmt )
      h = {}
      until fmt.skip /\}/ do
        bug! if fmt.empty?

        unless fmt.scan( /(\w+):/ ) then
          bug!
        end
        n = fmt[1]
        h[ n ] = scan_h( fmt )
      end
    end

    def scan_h( fmt )
      if    fmt.skip /\{/    then scan_hash  fmt
      elsif fmt.skip /\(/    then scan_array fmt
      elsif fmt.skip /(\w);/ then scan_t     fmt[1]
      else
        bug!
      end
    end

    def scan_t( val, t )
      case t
      when 's' then StringChk.new
      when 'r' then RegexpChk.new
      when 'a' then TypeChk.new( ArrayRcNode )
      when 'h' then TypeChk.new( HashRcNode )
      else
        bug!
      end
    end

    def scan_array( fmt )
      unless fmt.skip /(\d+)(?:(\*?)|-(\d+));/ then
        bug!
      end
      min = fmt[1]
      if fmt[2] then
        max = nil
      elsif max = fmt[3] then
        max = max.to_i
      else
        max = min
      end

      until fmt.skip /\)/ do
        if    fmt.skip /\{/    then arr.push scan_hash(  fmt    )
        elsif fmt.skip /\(/    then arr.push scan_array( fmt    )
        elsif fmt.scan /(\w);/ then arr.push scan_t(     fmt[1] )
        else
          bug!
        end
      end
    end


    class Chk
    
      def typechk( val, type )
        unless type === val then
          raise ConfigError,
            "wrong config value type #{v} required #{type}"
        end
      end
    
    end
        
    class StringChk < Chk

      def chk( val )
        chktype val, String
        eval '"' + val + '"'
      end
    
    end

    class RegexpChk < Chk

      def chk( val )
        chktype val, String
        Regexp.new val
      end
    
    end

    class TypeChk < Chk

      def initialize( type )
        @type = type
      end

      def chk( val )
        typechk val, @type
        val
      end
    
    end

    class ArrayChk < Chk

      def initialize( chks, min, max = nil )
        @chks = chks
        @min = min
        @max = max
      end

      def chk( ent )
        if not @max and @min <= 1 then
          if String === ent then
            # str[,str...]
            ent = ArrayRcNode.new( ent )
          end
        end
        typechk ent, ArrayRcNode

        if ent.size < @min then
          raise ConfigError,
            "too few number of values for config entry '#{key}'"
        end
        if @max and ent.size > @max then
          raise ConfigError,
            "too many number of values for config entry '#{key}'"
        end

        chks = @chks.dup
        unless @max then
          last = chks[-1]
          while chks.size < @ent.size do
            chks.push last
          end
        end
        chks.each_with_index do |c, idx|
          ent[idx] = c.chk( ent[idx] )
        end

        ent
      end
    
    end

    class HashChk < Chk

      def initialize( chks )
        @chks = chks
      end

      def chk( ent )
        typechk ent, HashRcNode

        @chks.each do |k,c|
          unless v = ent[k] then
            raise ConfigError, "config key '#{ent.name}.#{k}' does not exist"
          end
          ent[k] = c.chk( v )
        end

        ent
      end
    
    end

  end

end
