#
# getter.rb
#
#   Copyright (c) 1999,2000 Minero Aoki <aamine@dp.u-netsurf.ne.jp>
#
#   This program is free software.
#   You can distribute/modify this program under the terms of
#   the GNU Lesser General Public License version 2 or later.
#

require 'est/strategy'


module EST

  class SeenGetter < Strategy

    RC_DEFAULT = {}

    VALUE_TYPE = 'boolean'

    def get( mail )
      mail.port.unread
    end
  
  end


  class MultipartGetter < Strategy

    RC_DEFAULT = {}

    VALUE_TYPE = 'boolean'

    def get( mail )
      mail.multipart?
    end
  
  end


  class DateGetter < Strategy

    RC_DEFAULT = { 'format' => '%m/%d %H:%M' }

    VALUE_TYPE = 'string'

    def get( mail )
      if t = mail.date(nil) then
        t.strftime( @rc['format'] )
      else
        ''
      end
    end

  end


  class FromGetter < Strategy

    RC_DEFAULT = {}

    VALUE_TYPE = 'string'

    def get( mail )
      mail.from_phrase(nil) || mail.from('')
    end

  end


  class SubjectGetter < Strategy

    RC_DEFAULT = {
      'sub-regexp' => /\[[^:\]]+:(\d+)\]/
    }

    VALUE_TYPE = 'string'

    def get( mail )
      if str = mail.subject(nil) then
        re = @rc['sub-regexp']
        if String === re then
          @rc['sub-regexp'] = re = Regexp.compile( re )
        end
        if re and m = re.match( str ) then
          a = m.to_a
          str.sub! re, a[ 1, m.size - 1 ].join(' ')
        end
        str
      else
        ''
      end
    end

  end

end
