#
# readrc.rb
#
#   Copyright (c) 1999,2000 Minero Aoki <aamine@dp.u-netsurf.ne.jp>
#
#   This program is free software.
#   you can distribute/modify this program under the terms of
#   the GNU Lesser General Public License version 2 or later.
#

require 'est/confp'


module EST


class ConfigError < StandardError; end


class RcReader

  def RcReader.read( fn )
    ent = ConfigFileParser.parse( fn )
    reader = new
    reader.configure ent

    ent['first-profile']
  end

  ConfigClassList = %w(
    sender
    popper
    mailbox
    composer
    refiler
    getter
    profile
    address
    interface
  )

  def configure( entries )
    ConfigClassList.each do |ccn|
# puts "configuring #{ccn}..."
      mname = 'setup_' + ccn.downcase.gsub( '-', '_' )
      send mname, entries[ccn]
    end
  end


  def setup_interface( config )
    send 'setup_' + ::EST::Interface, config[ ::EST::Interface ]
  end

  def setup_sender( config )
    config.each do |name, rc|
      CORE.sender.add name, rc
    end
  end

  def setup_popper( config )
    config.each do |name, rc|
      CORE.popper.add name, rc
    end
  end

  def setup_mailbox( config )
    config.each do |name, rc|
      CORE.mailbox.add_mbox name, rc
    end
  end

  def setup_composer( config )
    config.each do |name, rc|
      CORE.composer.add name, rc
    end
  end

  def setup_refiler( config )
    return unless config

    config.each do |name, rc|
      rule = rc['rule']
unless rule
p rule
bug!
end
      rule.each do |ent|
        # ent = array of [ header, refile_regex, dest_mbox ]
        ent[1] = Regexp.new( ent[1] )
      end
      CORE.refiler.add name, rc
    end
  end

  def setup_profile( config )
    config.each do |name, rc|
      CORE.profile.add name, rc
    end
  end

  def confirm_config( profnm, profiles )
    ent = profiles[ profnm ]
    unless ent['mailbox'] and
           ent['sender'] and
           ent['popper'] and
           ent['composer']
      CORE.warn "EST Warning: profile #{profnm} not exist: delete entry"
      profiles.delete profnm
    end
  end

  def setup_getter( config )
    config.each do |nm, rc|
      CORE.getter.add nm, rc
    end
  end

  def setup_address( config )
    config.each do |name, rc|
      CORE.address.add name, rc
    end
  end

end

end   # module EST
