#
# ifcore.rb
#
#   Copyright (c) 1999,2000 Minero Aoki <aamine@dp.u-netsurf.ne.jp>
#
#   This program is free software.
#   you can distribute/modify this program under the terms of
#   the GNU Lesser General Public License version 2 or later.
#

require 'gtk'
require 'est/all'


module EST

  class Core < CCore

    def initialize( rc )
      super
      @setter = SetterManager.new

      @working       = false
      @shutting_down = false

      @editers = []
    end

    attr :setter
    attr :working

    def trigger
      return false if @working

      first_profile = construct
      @mailbox.init0

      CORE.profile.select first_profile
      @mailbox.init1

      @working = true
      notice "mailest ready"
      Gtk.main
    end

    def shutdown( force = false )
      return if @shutting_down
      return unless @working
      begin
        @shutting_down = true

        unless force then
          unless quit_ok? then
            return
          end

          @editers.each {|ed| ed.shutdown }
          @editers.clear
          super()
        end
      ensure
        @shutting_down = false
        @working = false
      end
    end


    attr :ifrc, true   # tmp

    def quit_ok?
      true         # tmp
    end
    private :quit_ok?


    def notice( str )
      if @working then
        @mailbox.write str
      else
        $stderr.puts str
      end
    end


    def edit( port )
      unless @shutting_down then
        rc = CORE.ifrc['editer']
        cls = (rc['type'] == 'internal') ? InternalEditer : ExternalEditer
        @editers.push cls.new( port, rc )
        return true
      else
        return false
      end
    end

    def edit_done( ed )
      unless @shutting_down then
        @editers.delete( ed )
      end
    end

    def editers
      @editers.dup
    end


    def send_from_draft( arr = nil )
      unless arr then
        arr = []
        mailbox.draft.each_selection {|i| arr.push i }
      end
      super( *arr )
    end

  end

end
