#
# setter.rb
#
#   Copyright (c) 1999,2000 Minero Aoki <aamine@dp.u-netsurf.ne.jp>
#
#   This program is free software.
#   you can distribute/modify this program under the terms of
#   the GNU Lesser General Public License version 2 or later.
#


module EST

  class SetterManager < AgentManager

    def init_after
      register_class 'pixmap', PixmapSetter
      register_class 'string', StringSetter
    end
    private :init_after

  end



  class MailestSetter < Setter
  
    def set( win, row, idx, mail )
      setv win, row, idx, @getter.get( mail )
    end

  end


  class StringSetter < MailestSetter

    VALUE_TYPE = 'string'
    RC_DEFAULT = {}
      
    def setv( win, row, idx, val )
      win.set_text row, idx, val
      val
    end
  
  end


  class PixmapSetter < MailestSetter

    VALUE_TYPE = 'boolean'
    RC_DEFAULT = {}

    def initialize( rc )
      super rc
      @init = false
      @t_pix = @t_mask = nil
      @f_pix = @f_mask = nil
    end

    def ready( getter, win )
      super getter
      unless @init then
        @init = true
        @t_pix, @t_mask = getpm( win, @rc['file'] )
        # @f_pix, @f_mask = getpm( win, pm_false )
      end
    end

    def getpm( win, fname )
# p win
# p win.realized?
# p win.window
# p fname
      if fname then
        begin
          path = CORE.search_file( fname, 'share' )
# puts "pixmap path=<#{path}>"
          return Gdk::Pixmap.create_from_xpm( win.window, nil, path )
        rescue
          bug! 'pixmap not found' if $DEBUG
          CORE.warn "can't load pixmap '#{fname}':\n#{$!.to_s}"
        end
      end
      return nil, nil
    end

    def setv( win, row, idx, val )
      if (val and @t_pix) or (!val and @f_pix) then
        win.set_pixmap( row, idx, val ? @t_pix : @f_pix,
                                  val ? @t_mask : @f_mask )
      else
        win.set_text row, idx, ''
      end
      val
    end

  end


end   # module EST


module Gtk

  class CTree

    alias set_text node_set_text
    alias set_pixmap node_set_pixmap
  
  end

end
